function I = FEM2D_quadrature(f, mesh)
% Compute an approximation to (f, phi_k)_{L^2(\Omega)} for all FEM basis
% functions phi_k by Gauss quadrature.
% It is assumed that the mesh is the standard triangularization of the unit
% square (in order to easily get the constant meshwidth h and not have to
% compute the areas of the triangles).
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
p = mesh.p;
t = mesh.t(1:3,:);
e = mesh.e;

h = p(1,2);

Nt = size(t, 2);

% Vectorized version:

p1 = p(:, t(1,:)); p1x = p1(1,:); p1y = p1(2,:);
p2 = p(:, t(2,:)); p2x = p2(1,:); p2y = p2(2,:);
p3 = p(:, t(3,:)); p3x = p3(1,:); p3y = p3(2,:);

xi1 = p1x + (p2x-p1x)*1/3 + (p3x-p1x)*1/3;
xi2 = p1y + (p2y-p1y)*1/3 + (p3y-p1y)*1/3;

Ivals = 1/2*h^2 * f(xi1, xi2) * 1/3;
% I2 = sparse([t(1,:); t(2,:); t(3,:)], 1, [Ivals; Ivals; Ivals])
I = sparse(reshape(t, [],1), 1, repmat(Ivals, 3, 1));


% What is really happening:
% I = zeros(size(p, 2), 1);
% for j = 1:Nt
%     tj = t(:,j);
%     p1 = p(:,tj(1)); p1x = p1(1); p1y = p1(2);
%     p2 = p(:,tj(2)); p2x = p2(1); p2y = p2(2);
%     p3 = p(:,tj(3)); p3x = p3(1); p3y = p3(2);
% 
%     J = [p2x - p1x, p3x - p1x; p2y - p1y, p3y - p1y];
%     
%     xi1 = p1x + (p2x-p1x)*1/3 + (p3x-p1x)*1/3;
%     xi2 = p1y + (p2y-p1y)*1/3 + (p3y-p1y)*1/3;
%     
%     It = 1/2*abs(det(J)) * f(xi1, xi2) * 1/3;
% 
%     I(tj) = I(tj) + It;
%     
% end



end