function v = L2projection_FEM_batched(u, IHh, MH, Mh)
% Project u \in V_h onto V_H using the L^2-orthogonal projection. 
% IHh: interpolation matrix from V_H to V_h
% MH: mass matrix for V_H
% Mh: mass matrix for V_h
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
N = size(u, 3);
v = zeros(size(MH, 1), size(u, 2), N);

if (N > 1) && (size(u, 2) > 1)
    for k = 1:N
        v(:,:,k) = MH\((IHh'*(Mh*u(:,:,k))));
    end
else
    v = MH\((IHh'*(Mh*u)));
end

end