function err = LVV_err_LDLT(L1, D1, L2, D2, M, K, LK)
% Supposing X1 = L1*D1*L1', X2 = L2*D2*L2'.
% We want to calculate ||LK'(X1 - X2)*M*/LK'||, corresponding to the 
% V-norm of the infinite-dimensional representation of X1-X2
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
if nargin < 7
    LK = ichol(-K, struct('type','ict','droptol',1e-6));
end

G1 = [LK'*L1, LK'*L2];
G2 = [LK\(M*L1), LK\(M*L2)];
D = blkdiag(D1, -D2);
% Now LK'*(X1 - X2)*M/LK' = G1 D G2'

[U, R1] = qr(G1, 0);
[V, R2] = qr(G2, 0);
S = R1*D*R2';
% Now LK'(X1 - X2)*M*/LK' = U*S*V' (and S is a small matrix)
% Hence ||X1-X2|| = norm(S). 

% Absolute error:
err = norm(S);

% Relative error
[U, R1] = qr(LK'*L1, 0);
[V, R2] = qr(LK\(M*L1), 0);
S = R1*D1*R2';

P2norm = norm(S);

err = err / P2norm;
    
end
