function dQW = QWiener1D_FEM_explicit(Nt, dt, Nx, a, X, Y, lambda, M, K, N, params, project, FEMdata)
% Use the explicit representations of N e_j to generate the boundary noise
% directly
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%

if (nargin < 12)
    project = false;
end

% Eigenvalues 1D
if isfield(params, 'l')
    l = params.l;
    mu_j = 2 ./ l ./ (pi^2*(1:Nx)'/a + 1/l^2);

elseif isfield(params, 'beta')
    beta = params.beta;
    eps = params.eps;
    mu_j = ((1:Nx)'.^2).^ (-(beta + eps));
   
elseif isfield(params, 'r')
    r = params.r;
    eps = params.eps;
    mu_j = (1:Nx)'.^(-(2*r+1+eps));
end

if ~project
    % N e_j for the first Nx eigenvectors e_j
    % scaled by sqrt(mu_j)
    Ne = zeros(Nx^2, Nx);
    for j = 1:Nx
        % What might look like wrong signs on Ne1 and Ne3 is because the
        % outward-pointing normal points in the negative direction on two
        % of the sides, and in the positive direction on the others
        Ne1 = cos(j*pi*X/a).*cosh(sqrt(lambda + j^2*pi^2/a^2)*(a-Y)) / sqrt(lambda + j^2*pi^2/a^2) / sinh(a*sqrt(lambda + j^2*pi^2/a^2));
        Ne2 = cos(j*pi/a*X).*cosh(sqrt(lambda + j^2*pi^2/a^2)*Y) / sqrt(lambda + j^2*pi^2/a^2) / sinh(a*sqrt(lambda + j^2*pi^2/a^2));
        Ne3 = cos(j*pi/a*Y).*cosh(sqrt(lambda + j^2*pi^2/a^2)*(a-X)) / sqrt(lambda + j^2*pi^2/a^2) / sinh(a*sqrt(lambda + j^2*pi^2/a^2));
        Ne4 = cos(j*pi/a*Y).*cosh(sqrt(lambda + j^2*pi^2/a^2)*X) / sqrt(lambda + j^2*pi^2/a^2) / sinh(a*sqrt(lambda + j^2*pi^2/a^2));

        Ne(:,j) = sqrt(mu_j(j))*reshape(Ne1 + Ne2 + Ne3 + Ne4, Nx^2, 1);
    end
else
    Ne = FEMdata.RhNv1 + FEMdata.RhNv2 + FEMdata.RhNv3 + FEMdata.RhNv4;
    Ne = Ne * diag(sqrt(mu_j));
    
end

% Gaussian noise
dW = randn(Nx, Nt, N) * sqrt(dt);

% dQW = Ne * dW;
dQW = zeros(Nx^2, Nt, N);
for k = 1:N
    dQW(:,:,k) = (lambda*M - K) * Ne * dW(:,:,k);
end

end