Description:
============
This code was used to generate the figures in the paper [BST20], which concerns the error analysis of a fully discrete scheme
for a LQR problem with both distributed and boundary controls, as well as both distributed and boundary noise. Note that since
the codebase grew organically, there are many things that may be confusing but which makes sense from a backwards compatibility
standpoint. We also acknowledge that the documentation is very sparse. This might be improved in the future if people other 
than the authors find the code useful. Until then, feel free to contact the second author in case of questions.

[BST20] P. Benner, T. Stillfjord, C. Trautwein, A linear implicit Euler method for the finite element discretization of a controlled stochastic heat equation, submitted.

Dependencies:
=============

* FreeFem++, v3.61.1: https://freefem.org/ (other versions are likely OK too, this one is available from the Ubuntu repositories)

* DREsplit, v20200629: http://www.tonystillfjord.net (zip-file in the Software section)

* Matlab Signal Toolbox  (inexplicably required for the dct function)

* matlab2tikz v.1.1.0: https://github.com/matlab2tikz/matlab2tikz (optional, but files need to be manually edited if not available) 


Workflow
========

* Generate the matrices arising from the finite element discretization of the problem by running

    FreeFem++ generate_problem.edp
  
  after changing the output directory on line 1 to the desired location (this location needs to exist). If problems with finer
  spatial discretizations than in [BST20] are needed, the number 128 (2^7) on line 3 needs to be increased.

* Then convert the output into Matlab-readable format by running

    FF_to_Matlab(path, number_of_distributed_inputs, number_of_outputs, cleanup)

  Setting the last parameter to true will remove the files originally generated by FreeFem++ to save space.
  The number of distributed inputs and outputs are problem-dependent. In the supplied problem definition, they are
  9 and 3, respectively.

* Set data locations, discretization parameters, noise parameters etc. in the corresponding setup_problem.m. (The "space", "time" and "spacetime" directories must exist, or the following scripts will fail.) 
  These files also contain the ability to run the FF_to_Matlab script directly if necessary, and this currently
  depends on the number of inputs and outputs having been set correctly (manually).

* Run the main test script

    main_test_spacetime_with_parameters(Lmax, N, Nbatch, reuse_DREsol)
  
  after modifying the relevant lines at the beginning. In particular, use e.g.

    data = setup_problem_spacetime(Lmax, true);
  
  to initialize the parameter structure defined in the previous step. The matrices are loaded automatically.

  If reuse_DREsol = false, this will solve the associated DRE and store the solution in the folder specified in the setup_problem script. On subsequent runs where only the initial condition and noise parameters change, reuse_DREsol can then be set to true.

  Important: Nbatch is the number of sample paths to compute simultaneously. Do not set this too high initially, because
  you might run out of RAM. The experiments in [BST20] were run with the parameters Lmax = 7, N = 100 and Nbatch = 10. 

  Various output data is generated, see the last two cells for exactly what is computed. The actual outputs and inputs
  are only computed for a single sample path, defined by the index 'plotsample'. The cost functionals cost_ref and cost_cref
  for the controlled and uncontrolled reference solutions are computed only over one batch (i.e. Nbatch sample paths).

* In order to generate Figure 3 in [BST20], run plot_main_convergence. To generate the same kind of plot for a different problem, 
  edit the first few lines.
  
* In order to generate Figures 1 and 2 in [BST20], first run the scripts test_DRE_temporal_convergence.m and
  test_DRE_spatial_convergence.m, respectively, to compute the relevant errors. These scripts utilize the setup_problem_time and
  setup_problem_space functions, which are similar to setup_problem_spacetime but use a fixed spatial or temporal discretization as
  described in [BST20]. Then run the scripts plot_DRE_temporal_convergence.m and plot_DRE_spatial_convergence.m,
  respectively, to actually generate the plots.
  
* The associated DRE may be solved manually (by DREsplit) by running compute_optimal_feedback.m. This is called by the functions in 
  the previous point.
   
* A solution trajectory can be animated by using the animate_solution script

