function n = RMS_L2_norm(Z, M)
% Compute sqrt( 1/N \sum_{j=1}^{N} { L2norm(Z(:,j))^2 }
% for Z with size(Z) = [Nx, N] 
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
N = size(Z, 2);

if N > 1000
    n = 0;
    for j = 1:N
    %     n = n + L2norm(Z(:,j), M)^2;
        n = n + Z(:,j)' * M * Z(:,j);
    end
    n = sqrt(n / N);
else
    % More efficient, if we can form a dense matrix of size N x N
    n = sqrt( trace(Z'*M*Z) / N );
end



end
