function J = cost_functional(Nt, dt, C, x, Q, LT, DT, R, K)
% Compute the value of the cost functional
%
% J(x) = int_0^T { (Q*C*x(t), C*X(t)) + (R*u(t), u(t)) dt } +
% (LT*DT*LT'*x(T), X(T)
% 
% where u(t) = -R^{-1}*B'*X(t)*M*x(t) = -K(t)*x(t)
% 
% If R and K are omitted, the (Ru, u)-part is set to zero
%
% Nt denotes the number of time steps in the time discretization and
% dt is the time step size.
%
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
J = 0;
for j = 1:Nt
    J = J + x(:,j)'*(C'*Q*(C*x(:,j)));
    if (nargin > 7) && ~(isscalar(R) && (R == 0))
%         ut = 1/R*(B'*L{j})*D{j}*(L{j}'*(M*x(:,j)));
        ut = K{j}*x(:,j);
        J = J + (R*ut)'*ut;
    end
end

J = dt*J + (x(:,end)'*LT)*DT*(LT'*x(:,end));

end