function [vertices, triangles, edges, boundaryedges] = loadmesh(filename, d)
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
if nargin < 2
    d = 2;
end

file = fopen(filename,'r');
line1 = fscanf(file, '%d', 3);

Nv = line1(1);
Nt = line1(2);
Nbe = line1(3);

vertices = zeros(Nv, d);
for j = 1:Nv
    line = fscanf(file, '%f', d+1);
    vertices(j, :) = line(1:d)';
end

triangles = zeros(Nt, 3);
for j = 1:Nt
    line = fscanf(file, '%f', 4);
    triangles(j, :) = line(1:3)';
end

boundaryedges = zeros(Nbe, 3);
for j = 1:Nbe
    line = fscanf(file, '%f', 3);
    boundaryedges(j, :) = line';
end

% Find all edges
rawedges = reshape([triangles(:,1:2), triangles(:,2:3), triangles(:,3), triangles(:,1)]', 2, [])';
factor = 2^ceil(log2(Nv)) + 1;

% Convert each edge into a unique identifier. The integer part is the
% smallest vertex index and the fraction is the largest index divided by
% factor. (Inspired by code from Fredrik Hellman.)
ue = unique(min(rawedges, [], 2) + max(rawedges, [], 2) / factor);
edges = [floor(ue), floor((mod(ue,1))*factor)];

ne = size(edges, 1);

% Add some fictional data for the pdetoolbox
edges = [edges, zeros(ne, 1), ones(ne, 1), ones(ne, 1), ones(ne,1)];
triangles(:,4) = 1;

% Transpose for pdetoolbox
vertices = vertices';
triangles = triangles';
boundaryedges = boundaryedges';
edges = edges';


end