function data = setup_problem_space(Lmax, convert_matrices, local)
% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%
if nargin < 3
    local = false;
end

if nargin < 2
    convert_matrices = false;
end

if nargin < 1
    Lmax = 5;
end

if ~local
    data.path = './data_problem/';
    data.outputpath = './space/';
    data.name = 'problem_space';
else
    data.path = 'D:\\spde_data\\data_problem\\';
    data.outputpath = 'D:\\spde_data\\space\\';
end

data.noutputs = 3; % Number of outputs
data.ndinputs = 9; % Number of distributed inputs
data.nbinputs = 1; % Number of boundary inputs

if convert_matrices
    if ~exist([data.path, 'data4.mat'], 'file') % has already been generated
        fprintf('Converting finite element matrices to MATLAB format, and creating meshes...\n')
        FF_to_Matlab(data.path, data.ndinputs, data.noutputs);
        fprintf('...done.\n')
    end
end

data.Ls = 1:Lmax;

data.T = 2; % Terminal point of time

Nt = zeros(Lmax, 1);
dt = zeros(Lmax, 1);
tk{Lmax} = 0;

Nx = zeros(Lmax, 1);
dx = zeros(Lmax, 1);
x{Lmax} = 0;
X{Lmax} = 0;
Y{Lmax} = 0;
a = 1;

for L = data.Ls
    % Temporal discretizations
    Nt(L) = 2^(9);
    dt(L) = data.T / Nt(L);
    tk{L} = linspace(0, data.T, Nt(L)+1);

    % Spatial discretization
    Nx(L) = 2^(L);
    dx(L) = 1/Nx(L);
    x{L} = linspace(0, a, Nx(L)+1)';
    [X{L}, Y{L}] = meshgrid(x{L}, x{L});
end

data.L = L;
data.Nt = Nt;
data.dt = dt;
data.tk = tk;
data.Nx = Nx;
data.dx = dx;
data.x = x;
data.X = X;
data.Y = Y;
data.a = 1;

data.lambda = 1;
data.diff_coeff = 1e-2;

%% Cost functional weights
% Output weights
data.Q = 1e4*eye(data.noutputs, data.noutputs);

% Input weights
data.RD = 1e-2*diag(ones(data.ndinputs, 1));
data.RN = 2500*1e-2*diag(ones(data.nbinputs, 1));
data.R = blkdiag(data.RD, data.RN);

% Final state penalizer
data.RT = 0;

end