% Copyright 2020 Tony Stillfjord
%
% This file is part of BST20_CODE.
% 
% BST20_CODE is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% BST20_CODE is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with BST20_CODE.  If not, see <http://www.gnu.org/licenses/>.
%

clear all
addpath('../DREsplit')

Lmax = 8;
% data = setup_problem_time(Lmax, true); % Mechthild
data = setup_problem_time(Lmax, true, true); % local

reuse_DREsol = false;
% reuse_DREsol = true;

% Load matrices
FEMdata = load([data.path, 'data', int2str(data.Nx(end)), '.mat']);

% Compute optimal feedback by solving DRE
if ~reuse_DREsol
    DREdata = compute_optimal_feedback(data, FEMdata, true);
else
    DREdata = load([data.outputpath, 'DREsol_', data.name, '_Nx_', int2str(data.Nx(end)), '_Nt_', int2str(data.Nt(end)), '.mat']);
end

Lref = DREdata.Lss{Lmax}{end};
Dref = DREdata.Dss{Lmax}{end};

M = FEMdata.M;
LM_inc = ichol(M,struct('type','ict','droptol',1e-6));
B = FEMdata.BN;

Xref_norm = LHH_norm_LDLT(Lref, Dref, M, LM_inc);
BXref_norm = norm( ((B'*Lref*Dref*Lref') * LM_inc) );

errs_X_abs = zeros(Lmax-1,1);
errs_BX_abs = zeros(Lmax-1,1);

for k = 1:Lmax-1
    Lk = DREdata.Lss{k}{end};
    Dk = DREdata.Dss{k}{end};
    
    errs_X_abs(k) = LHH_err_LDLT(Lk, Dk, Lref, Dref, M, LM_inc);
    errs_BX_abs(k) = norm( ((B'*Lk*Dk*Lk' - B'*Lref*Dref*Lref' ) * LM_inc));
end

errs_X_rel = errs_X_abs / Xref_norm;
errs_BX_rel = errs_BX_abs / BXref_norm;

%%
results.errs_X_abs = errs_X_abs;
results.errs_X_rel = errs_X_rel;
results.errs_BX_abs = errs_BX_abs;
results.errs_BX_rel = errs_BX_rel;

save([data.outputpath, 'results_', int2str(data.Nt(end)), '_', int2str(data.Nx(end)), '.mat'], '-struct', 'results', '-v7.3')